///////////////////////////////////////////////////////////////////////
// RtosCppDemoCtl.cpp 
//
// Implementation of the CRtosDemoCtrl ActiveX Control class.

#include "stdafx.h"
#include "RtosCppDemo.h"
#include "RtosCppDemoCtl.h"
#include "RtosDemoPpg.h"
#include "nertos_i.c"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char* TaskStateName[] = 
{
	"Suspended",
	"Waiting",
	"Waiting for Semaphore",
	"Running"
};

char* TimerStateName[] =
{
	"Stopped",
	"Running"
};

#define GUID_CCH		39  // Characters in string form of guid, including '\0'
#define MAXMAXTASKS		100
#define MAXMAXSEMAS		100
#define MAXMAXTIMERS	100
#define IDLETASKNAME	"Idle"
#define EMULGOMSG		"Emulator Running"
#define EMULBREAKMSG	"Emulator Stopped"
#define TRACESTARTMSG	"Trace Started"
#define	TRACESTOPMSG	"Trace Stopped"
#define	RTKKEYNAME		"Embedded RTOS debug module"

IMPLEMENT_DYNCREATE(CRtosDemoCtrl, CFormControl)

/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CRtosDemoCtrl, CFormControl)
	//{{AFX_MSG_MAP(CRtosDemoCtrl)
	ON_BN_CLICKED(IDC_GO, OnGo)
	ON_BN_CLICKED(IDC_BREAK, OnBreak)
	ON_BN_CLICKED(IDC_START, OnStart)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_NOTIFY(TCN_SELCHANGE, IDC_TABCONTROL, OnSelchangeTabcontrol)
	ON_WM_SETFOCUS()
	ON_WM_LBUTTONUP()
	ON_WM_KILLFOCUS()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CRtosDemoCtrl, CFormControl)
	//{{AFX_DISPATCH_MAP(CRtosDemoCtrl)
	// NOTE - ClassWizard will add and remove dispatch map entries
	//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_DISPATCH_MAP
	DISP_FUNCTION_ID(CRtosDemoCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()

/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CRtosDemoCtrl, CFormControl)
	//{{AFX_EVENT_MAP(CRtosDemoCtrl)
	// NOTE - ClassWizard will add and remove event map entries
	//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_EVENT_MAP
END_EVENT_MAP()

/////////////////////////////////////////////////////////////////////////////
// Interface map

BEGIN_INTERFACE_MAP(CRtosDemoCtrl, CFormControl)
    INTERFACE_PART(CRtosDemoCtrl, IID_INertos, NertosObj)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CRtosDemoCtrl, 1)
	PROPPAGEID(CRtosDemoPropPage::guid)
END_PROPPAGEIDS(CRtosDemoCtrl)

/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CRtosDemoCtrl, "RTKDEMO.RtosDemoCtrl.1",
	0x4f4efc6, 0xf749, 0x11d2, 0x87, 0x3b, 0x44, 0x45, 0x53, 0x54, 0, 0)

/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CRtosDemoCtrl, _tlid, _wVerMajor, _wVerMinor)

/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DRtosDemo =
		{ 0x4f4efc4, 0xf749, 0x11d2, { 0x87, 0x3b, 0x44, 0x45, 0x53, 0x54, 0, 0 } };
const IID BASED_CODE IID_DRtosDemoEvents =
		{ 0x4f4efc5, 0xf749, 0x11d2, { 0x87, 0x3b, 0x44, 0x45, 0x53, 0x54, 0, 0 } };

/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwRtosDemoOleMisc =
	OLEMISC_ACTIVATEWHENVISIBLE |
	OLEMISC_SETCLIENTSITEFIRST |
	OLEMISC_INSIDEOUT |
	OLEMISC_CANTLINKINSIDE |
	OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CRtosDemoCtrl, IDS_RTOSDEMO, _dwRtosDemoOleMisc)

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::CRtosDemoCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CRtosDemoCtrl

BOOL CRtosDemoCtrl::CRtosDemoCtrlFactory::UpdateRegistry(BOOL bRegister)
{
	USES_CONVERSION;

	// TODO: Verify that your control follows apartment-model threading rules.
	// Refer to MFC TechNote 64 for more information.
	// If your control does not conform to the apartment-model rules, then
	// you must modify the code below, changing the 6th parameter from
	// afxRegApartmentThreading to 0.

	if (bRegister)
	{
		BOOL bSuccess = AfxOleRegisterControlClass(
			AfxGetInstanceHandle(),
			m_clsid,
			m_lpszProgID,
			IDS_RTOSDEMO,
			IDB_RTOSDEMO,
			afxRegApartmentThreading,
			_dwRtosDemoOleMisc,
			_tlid,
			_wVerMajor,
			_wVerMinor);

		if (!bSuccess)
			return FALSE;

		// Format class ID as a string
		OLECHAR szClassID[GUID_CCH];
		int cchGuid = ::StringFromGUID2(m_clsid, szClassID, GUID_CCH);
		LPCTSTR lpszClassID = OLE2CT(szClassID);

		// Did StringFromGUID2 work?
		ASSERT(cchGuid == GUID_CCH);
		if (cchGuid != GUID_CCH)
			return FALSE;

		// Attempt to create RTK key under CLSID key
		HKEY hkey = NULL;
		TCHAR szScratch[_MAX_PATH];
		wsprintf(szScratch, _T("CLSID\\%s\\%s"), lpszClassID, RTKKEYNAME);
		if (::RegCreateKey(HKEY_CLASSES_ROOT, szScratch, &hkey) != ERROR_SUCCESS)
			return FALSE;

		ASSERT(hkey != NULL);
		::RegCloseKey(hkey);
		return TRUE;
	}
	else
		return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::CRtosDemoCtrl - Constructor

CRtosDemoCtrl::CRtosDemoCtrl()
	: CFormControl(CRtosDemoCtrl::IDD, TRUE)
{
	//{{AFX_DATA_INIT(CRtosDemoCtrl)
	m_strStatus = _T("");
	//}}AFX_DATA_INIT

	InitializeIIDs(&IID_DRtosDemo, &IID_DRtosDemoEvents);

	m_pEvents		= NULL;
	m_bRTOSInit		= false;
	m_nAppAddr		= 0;

	memset(&m_App, 0, sizeof(m_App));
	memset(&m_Task, 0, sizeof(m_Task));
	memset(&m_Timer, 0, sizeof(m_Timer));
	memset(&m_Sema, 0, sizeof(m_Sema));

	/*
	m_ambientDispDriver.AttachDispatch(&m_ambientDisp);
	*/
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::~CRtosDemoCtrl - Destructor

CRtosDemoCtrl::~CRtosDemoCtrl()
{
	if (m_pEvents)
	{
		m_pEvents->Release();
		m_pEvents = NULL;
	}
}

/////////////////// OnInitialUpdate()
//
// Initializes child controls.

void CRtosDemoCtrl::OnInitialUpdate()
{
	CFormControl::OnInitialUpdate();

	// Initialize tab strip
	TC_ITEM item;
	item.mask = TCIF_TEXT;
	item.pszText = "Tasks";
	m_tabControl.InsertItem(0, &item);
	item.pszText = "Timers";
	m_tabControl.InsertItem(1, &item);
	item.pszText = "Semaphores";
	m_tabControl.InsertItem(2, &item);
	item.pszText = "Graph";
	m_tabControl.InsertItem(3, &item);

	// Create the other two edit controls
	CRect rc;
	m_tasksEdit.GetWindowRect(&rc);
	ScreenToClient(&rc);
	DWORD dwStyle = m_tasksEdit.GetStyle();
	DWORD dwExStyle = m_tasksEdit.GetExStyle();

	VERIFY(m_timersEdit.CreateEx(dwExStyle, "EDIT", NULL, dwStyle, rc, 
		this, IDC_TIMERS_EDIT, NULL));
	m_timersEdit.SetFont(m_tasksEdit.GetFont());
	VERIFY(m_semaphoresEdit.CreateEx(dwExStyle, "EDIT", NULL, dwStyle, rc, 
		this, IDC_SEMAPHORES_EDIT, NULL));
	m_semaphoresEdit.SetFont(m_tasksEdit.GetFont());

	m_timersEdit.ShowWindow(SW_HIDE);
	m_semaphoresEdit.ShowWindow(SW_HIDE);
}

void CRtosDemoCtrl::DoDataExchange(CDataExchange* pDX)
{
	//{{AFX_DATA_MAP(CRtosDemoCtrl)
	DDX_Control(pDX, IDC_TASKS_EDIT, m_tasksEdit);
	DDX_Control(pDX, IDC_TABCONTROL, m_tabControl);
	DDX_Control(pDX, IDC_STOP, m_btnStop);
	DDX_Control(pDX, IDC_START, m_btnStart);
	DDX_Control(pDX, IDC_GO, m_btnGo);
	DDX_Control(pDX, IDC_BREAK, m_btnBreak);
	DDX_Text(pDX, IDC_STATUS_STATIC, m_strStatus);
	DDX_Text(pDX, IDC_TSTATUS_STATIC, m_strTStatus);
	DDX_Text(pDX, IDC_EMULATOR_STATIC, m_strEmulator);
	DDX_Text(pDX, IDC_MSG_STATIC, m_strMsg);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::OnDraw - Drawing function

void CRtosDemoCtrl::OnDraw(CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
	CBrush* pBrush = CBrush::FromHandle(::GetSysColorBrush(COLOR_BTNFACE));
	CBrush* pOldBrush = pdc->SelectObject(pBrush);
	pdc->FillRect(rcBounds, pBrush);
	pdc->SelectObject(pOldBrush);

	/* From MFC sample...

	if (!m_hWnd)
		CFormControl::OnDraw(pdc, rcBounds, rcInvalid);
	else
	{
		CRect rc(rcBounds);
		CPen* pOldPen = (CPen*)pdc->SelectStockObject(BLACK_PEN);
		pdc->DrawEdge(rc, EDGE_ETCHED, BF_RECT);
		pdc->SelectObject(pOldPen);
	}
	*/
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::DoPropExchange - Persistence support

void CRtosDemoCtrl::DoPropExchange(CPropExchange* pPX)
{
	ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
	CFormControl::DoPropExchange(pPX);

	// TODO: Call PX_ functions for each persistent custom property.

}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::OnResetState - Reset control to default state

void CRtosDemoCtrl::OnResetState()
{
	CFormControl::OnResetState();  // Resets defaults found in DoPropExchange

	// TODO: Reset any other control state here.
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl::AboutBox - Display an "About" box to the user

void CRtosDemoCtrl::AboutBox()
{
	CDialog dlgAbout(IDD_ABOUTBOX_RTOSDEMO);
	dlgAbout.DoModal();
}

void CRtosDemoCtrl::Append(CEdit& rEdit, CString str)
{
	int nCount  = rEdit.GetLineCount();
	int nIndex  = rEdit.LineIndex(nCount - 1);
	int nLength = rEdit.LineLength(nIndex);

	rEdit.SetSel(nIndex + nLength, nIndex + nLength);
	rEdit.ReplaceSel(str + "\r\n");
}

void CRtosDemoCtrl::InitRTOS()
{
	BSTR bstrArg;
	BSTR bstr1;
	BSTR bstr2;

	bstrArg = SysAllocString(L"View_Evaluate \"App\"");
	ASSERT(bstrArg);

	ASSERT(m_pEvents);
	m_pEvents->ExecCmd(bstrArg, &bstr1);
	SysFreeString(bstrArg);

	bstrArg = SysAllocString(L"Wnd_GetErr");
	m_pEvents->ExecCmd(bstrArg, &bstr2);
	SysFreeString(bstrArg);

	if (_wtoi(bstr2) == 0)
	{
		m_strMsg = "";
		m_bRTOSInit = true;
		m_nAppAddr = _wtoi(bstr1);
	}
	else
	{
		m_bRTOSInit = false;
		m_strMsg = "No symbols loaded!";
    }

	UpdateData(FALSE);
	SysFreeString(bstr1);
	SysFreeString(bstr2);
}

void CRtosDemoCtrl::UpdateAll()
{
	m_strMsg = "";
	if (m_bRTOSInit)
	{
		UpdateApp();
		if (!UpdateTasks() 
			|| !UpdateTimers()
			|| !UpdateSemaphores())
		{
			m_bRTOSInit = false;

			m_tasksEdit.SetWindowText("");
			m_timersEdit.SetWindowText("");
			m_semaphoresEdit.SetWindowText("");
		}
	}
	UpdateData(FALSE);
}

void CRtosDemoCtrl::UpdateApp()
{
	ASSERT(m_pEvents);
	m_pEvents->ReadMemory(0, m_nAppAddr, sizeof(m_App), (long) &m_App);
}

#define MAXCHARINDEX 99

CString CRtosDemoCtrl::GetStr16(int nameptr16)
{
	char nma[MAXCHARINDEX+1];
	CString nm;

	if (nameptr16 == 0) // null name pointer
		nm = "<null name pointer>";
	else
	{
		ASSERT(m_pEvents);
		m_pEvents->ReadMemory(0, nameptr16, sizeof(nma), (long)nma);
		nma[MAXCHARINDEX] = 0; // force the string to end
		nm = nma;
		if (!nm.GetLength())
			nm = "<no name>";
	}

	return nm;
}

bool CRtosDemoCtrl::UpdateTasks()
{
	bool res;
	int index;
	CString str;

	res = true; // Start out Good
	if (m_App.MaxTasks > MAXMAXTASKS) 
	{
		res = false;
		m_strMsg = "MaxTasks too large!";
	}
	else 
	{
		m_tasksEdit.SetWindowText("");
		index = 0;
		while (res && (index < m_App.MaxTasks)) 
		{
			if (m_App.TaskArray[index].Handle != 0) 
			{
				str = GetStr16(m_App.TaskArray[index].Name);
				if ((index == 0) && (str != IDLETASKNAME)) 
				{
					res = false;
					m_strMsg = "Incorrect Idle Task!";
				}
				else 
				{
					str = str + " - "
						+ TaskStateName[m_App.TaskArray[index].State];
					if (m_App.TaskArray[index].Timer != 0) 
					{
						ASSERT(m_pEvents);
						m_pEvents->ReadMemory(0, m_App.TaskArray[index].Timer,
							sizeof(m_Timer), (long)&m_Timer);
						str = str + ", Timer: " + GetStr16(m_Timer.Name);
					}
				}

				if (m_App.TaskArray[index].Semaphore != 0)
				{
					m_pEvents->ReadMemory(0, m_App.TaskArray[index].Semaphore,
						sizeof(m_Sema), (long) &m_Sema);
					str = str + ", Semaphore: " + GetStr16(m_Sema.Name);
				}

				Append(m_tasksEdit, str);
			}

			index++;
		} 
	} 

	UpdateData(FALSE);
	return res;
}

bool CRtosDemoCtrl::UpdateTimers()
{
	bool res;
	int index;
	CString str;

	res = true; // Start out Good
	if (m_App.MaxTimers > MAXMAXTIMERS) 
	{
		res = false;
		m_strMsg = "MaxTimers too large!";
	}
	else 
	{
		m_timersEdit.SetWindowText("");
		for (index = 0; index < m_App.MaxTimers; index++)
		{
			if (m_App.TimerArray[index].Handle != 0) 
			{
				char buf[100];

				str = GetStr16(m_App.TimerArray[index].Name)
					+ " - "
					+ TimerStateName[m_App.TimerArray[index].State]
					+ ", " + itoa(m_App.TimerArray[index].Cnt, buf, 10) + " ["
					+ itoa(m_App.TimerArray[index].Value, buf, 10) + "]";

				if (m_App.TimerArray[index].Task != 0) 
				{
					ASSERT(m_pEvents);
					m_pEvents->ReadMemory(0, m_App.TimerArray[index].Task,
						sizeof(m_Task), (long) &m_Task);
					str = str + ", Task: " + GetStr16(m_Task.Name);
				}

				Append(m_timersEdit, str);
			}
        } 
	}
 
	UpdateData(FALSE);
	return res;
}

bool CRtosDemoCtrl::UpdateSemaphores()
{
	bool res;
	int index, index2, nm;
	CString str;

	res = true; // Start out Good
	if (m_App.MaxSemas > MAXMAXSEMAS)
	{
		res = false;
		m_strMsg = "MaxSemas too large!";
	}
	else
	{
		m_semaphoresEdit.SetWindowText("");

		for (index = 0; index < m_App.MaxSemas; index++) 
		{
			if (m_App.SemaArray[index].Handle != 0) 
			{
				char buf[100];

				str = GetStr16(m_App.SemaArray[index].Name) + ", "
					+ itoa(m_App.SemaArray[index].Value, buf, 10) + " ["
					+ itoa(m_App.SemaArray[index].MaxValue, buf, 10) + "]";

				for (index2 = 0; index2 < MAXTASKS; index2++) 
				{
					if (m_App.SemaArray[index].Queue[index2] != 0) 
					{
						ASSERT(m_pEvents);
						m_pEvents->ReadMemory(0, 
							m_App.SemaArray[index].Queue[index2], 
							sizeof(nm), (long) &nm);
						str = str + ", " + GetStr16(nm);
					}
				}
				Append(m_semaphoresEdit, str);
			}
		}
	}

	UpdateData(FALSE);
	return res;
}

/////////////////////////////////////////////////////////////////////////////
// INertos implementation

ULONG FAR EXPORT CRtosDemoCtrl::XNertosObj::AddRef()
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	return pThis->ExternalAddRef();
}

ULONG FAR EXPORT CRtosDemoCtrl::XNertosObj::Release()
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	return pThis->ExternalRelease();
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::QueryInterface(
	REFIID iid, void FAR* FAR* ppvObj)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	return (HRESULT)pThis->ExternalQueryInterface(&iid, ppvObj);
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::ConnectEvents(INertosEvents* ine)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	if (pThis->m_pEvents)
		pThis->m_pEvents->Release();

	pThis->m_pEvents = ine;
	if (pThis->m_pEvents)
		pThis->m_pEvents->AddRef();

	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::Endian(short LEndian)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::ProgramLoaded(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->InitRTOS();
	pThis->UpdateAll();
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::ProgramReset(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->InitRTOS();
	pThis->UpdateAll();
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::ReadMemoryTest(short memtype, long address, long size, long dest)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::Init(BSTR initstring)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)

	pThis->m_strEmulator = CString(initstring);
	pThis->m_strStatus = EMULBREAKMSG;
	pThis->m_strTStatus = TRACESTOPMSG;
	pThis->UpdateData(FALSE);

	pThis->InitRTOS();
	pThis->UpdateAll();

	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::Terminate(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->m_strMsg = "";
	pThis->UpdateData(FALSE);
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::Started(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->m_strStatus = EMULGOMSG;
	pThis->UpdateData(FALSE);
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::Stopped(long pc)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)

	char buf[100];
	sprintf(buf, "%s at: %XH", EMULBREAKMSG, pc);
	pThis->m_strStatus = buf;
	pThis->UpdateData(FALSE);
	pThis->InitRTOS();
	pThis->UpdateAll();

	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::AboutBox(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->AboutBox();
	pThis->Append(pThis->m_tasksEdit, CString("task test"));
	pThis->Append(pThis->m_timersEdit, CString("timers test"));
	pThis->Append(pThis->m_semaphoresEdit, CString("semaphores test"));
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::TraceStarted(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->m_strTStatus = TRACESTARTMSG;
	pThis->UpdateData(FALSE);
	return S_OK;
}

HRESULT FAR EXPORT CRtosDemoCtrl::XNertosObj::TraceStopped(void)
{
	METHOD_PROLOGUE(CRtosDemoCtrl, NertosObj)
	pThis->m_strTStatus = TRACESTOPMSG;
	pThis->UpdateData(FALSE);
	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CRtosDemoCtrl message handlers

void CRtosDemoCtrl::OnGo() 
{
	if (m_pEvents)
		m_pEvents->Start();
}

void CRtosDemoCtrl::OnBreak() 
{
	if (m_pEvents)
		m_pEvents->Stop();
}

void CRtosDemoCtrl::OnStart() 
{
	if (m_pEvents)
		m_pEvents->TraceStart();
}

void CRtosDemoCtrl::OnStop() 
{
	if (m_pEvents)
		m_pEvents->TraceStop();
}

void CRtosDemoCtrl::OnSelchangeTabcontrol(NMHDR* pNMHDR, LRESULT* pResult) 
{
	switch (m_tabControl.GetCurSel())
	{
	case 0: // Tasks
		m_timersEdit.ShowWindow(SW_HIDE);
		m_semaphoresEdit.ShowWindow(SW_HIDE);
		m_tasksEdit.ShowWindow(SW_SHOWNORMAL);
		break;
	case 1: // Timers
		m_tasksEdit.ShowWindow(SW_HIDE);
		m_semaphoresEdit.ShowWindow(SW_HIDE);
		m_timersEdit.ShowWindow(SW_SHOWNORMAL);
		break;
	case 2: // Semaphores
		m_tasksEdit.ShowWindow(SW_HIDE);
		m_timersEdit.ShowWindow(SW_HIDE);
		m_semaphoresEdit.ShowWindow(SW_SHOWNORMAL);
		break;
	case 3: // Graph
		m_tasksEdit.ShowWindow(SW_HIDE);
		m_timersEdit.ShowWindow(SW_HIDE);
		m_semaphoresEdit.ShowWindow(SW_HIDE);
		break;
	}

	*pResult = 0;
}

void CRtosDemoCtrl::SizeTabs()
{
	CRect rcWnd;
	GetWindowRect(&rcWnd);
	ScreenToClient(&rcWnd);

	// Size tabs
	if (m_tabControl.m_hWnd)
	{
		CRect rcTabs;
		m_tabControl.GetWindowRect(rcTabs);
		ScreenToClient(&rcTabs);
		int nInc = rcTabs.left;
		m_tabControl.SetWindowPos(NULL, 0, 0, 
			max(1, rcWnd.right - 2 * nInc),
			max(1, rcWnd.bottom - rcTabs.top - nInc), 
			SWP_NOMOVE|SWP_NOOWNERZORDER);

		// Size edit controls
		CRect rcEdit;
		if (m_tasksEdit.m_hWnd)
		{
			m_tasksEdit.GetWindowRect(&rcEdit);
			ScreenToClient(&rcEdit);
			m_tasksEdit.SetWindowPos(NULL, 0, 0,
				max(1, rcWnd.right - 4 * nInc),
				max(1, rcWnd.bottom - rcEdit.top - 2 * nInc),
				SWP_NOMOVE|SWP_NOOWNERZORDER);

			if (m_timersEdit.m_hWnd)
			{
				m_timersEdit.SetWindowPos(NULL, 0, 0,
					max(1, rcWnd.right - 4 * nInc),
					max(1, rcWnd.bottom - rcEdit.top - 2 * nInc),
					SWP_NOMOVE|SWP_NOOWNERZORDER);
			}

			if (m_semaphoresEdit.m_hWnd)
			{
				m_semaphoresEdit.SetWindowPos(NULL, 0, 0,
					max(1, rcWnd.right - 4 * nInc),
					max(1, rcWnd.bottom - rcEdit.top - 2 * nInc),
					SWP_NOMOVE|SWP_NOOWNERZORDER);
			}
		}
	}
}

void CRtosDemoCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CFormControl::OnSize(nType, cx, cy);
	SizeTabs();	
}

/////////////////// _GetClippingCoordinates()
//
// Copied from MFC implementation source (ctlinplc.cpp).

void AFXAPI _GetClippingCoordinates(LPCRECT pPosRect, LPCRECT pClipRect,
	LPRECT pIntersectRect, LPPOINT pOffsetPoint)
{
	int clipLeft = 0;
	int clipTop = 0;

	if ((pClipRect == NULL) || IsRectEmpty(pClipRect))
	{
		CopyRect(pIntersectRect, pPosRect);
	}
	else
	{
		IntersectRect(pIntersectRect, pPosRect, pClipRect);
		clipLeft = pClipRect->left;
		clipTop = pClipRect->top;
	}

	pOffsetPoint->x = min(0, pPosRect->left - clipLeft);
	pOffsetPoint->y = min(0, pPosRect->top - clipTop);
}

/////////////////// OnActivateInPlace()
//
// Copied from MFC implementation source. 
//
// This is a duplicate of the MFC OnActivateInPlace with the creation of 
// the resize handles commented off.

HRESULT CRtosDemoCtrl::OnActivateInPlace(BOOL bUIActivate, LPMSG pMsg)
{
#ifdef _AFXDLL
	if (m_bOpen)
	{
		m_pWndOpenFrame->SetActiveWindow();
		SendAdvise(OBJECTCODE_SHOWWINDOW);
		return S_OK;
	}
#endif

	// Initialize pointer to in-place site, if necessary.
	if (m_pInPlaceSite == NULL)
	{
		if (m_pClientSite == NULL)
			return E_UNEXPECTED;

		if ((GetControlFlags() & windowlessActivate) &&
			SUCCEEDED(m_pClientSite->QueryInterface(IID_IOleInPlaceSiteWindowless,
			reinterpret_cast<void**>(&m_pInPlaceSiteWndless))))
		{
			m_bInPlaceSiteWndless = m_bInPlaceSiteEx = TRUE;
		}
		else if ((GetControlFlags() & noFlickerActivate) &&
			SUCCEEDED(m_pClientSite->QueryInterface(IID_IOleInPlaceSiteEx,
			reinterpret_cast<void**>(&m_pInPlaceSiteEx))))
		{
			m_bInPlaceSiteEx = TRUE;
		}
		else if (SUCCEEDED(m_pClientSite->QueryInterface(IID_IOleInPlaceSite,
			reinterpret_cast<void**>(&m_pInPlaceSite))))
		{
			m_bInPlaceSiteEx = FALSE;
		}
		else
		{
			m_pInPlaceSite = NULL;
			return E_FAIL;
		}
	}

	ASSERT(m_pInPlaceSite != NULL);

	if ((m_bInPlaceActive && !bUIActivate) || m_bUIActive)
	{
		CWnd* pWndOuter = GetOuterWindow();
		HWND hwndParent;
		if ((pWndOuter != NULL) &&
			SUCCEEDED(m_pInPlaceSite->GetWindow(&hwndParent)) &&
			(hwndParent == ::GetParent(pWndOuter->m_hWnd)))
		{
			::SetWindowPos(pWndOuter->m_hWnd, NULL, 0, 0, 0, 0,
				SWP_NOZORDER|SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|
				SWP_SHOWWINDOW);
			OnSetObjectRects(m_rcPos, NULL);
			return S_OK;
		}
	}

	// Check if container allows windowless activation.
	if (m_bInPlaceSiteWndless)
	{
		if (m_pInPlaceSiteWndless->CanWindowlessActivate() != S_OK)
			m_bInPlaceSiteWndless = FALSE;
	}

	HRESULT hr = E_FAIL;
	if (m_pInPlaceSite != NULL)
		hr = m_pInPlaceSite->CanInPlaceActivate();

	if (hr != NOERROR)
	{
		// Site doesn't allow in-place activation.
		return OnOpen(FALSE, pMsg);
	}

	if (!m_bInPlaceActive)
	{
		if (m_bInPlaceSiteEx)
		{
			// flicker-free and/or windowless activation
			BOOL bNoRedraw;
			m_pInPlaceSiteEx->OnInPlaceActivateEx(&bNoRedraw,
				m_bInPlaceSiteWndless ? ACTIVATE_WINDOWLESS : 0);
			if (GetControlFlags() & noFlickerActivate)
				m_bNoRedraw = bNoRedraw;
		}
		else
		{
			// old-style activation
			m_pInPlaceSite->OnInPlaceActivate();
		}
	}

	HWND hwndParent = NULL;

	if (SUCCEEDED(m_pInPlaceSite->GetWindow(&hwndParent)))
	{
		CRect rcClip;
		m_frameInfo.cb = sizeof(OLEINPLACEFRAMEINFO);

		RELEASE(m_pInPlaceFrame);
		RELEASE(m_pInPlaceDoc);

		if (SUCCEEDED(m_pInPlaceSite->GetWindowContext(
						&m_pInPlaceFrame, &m_pInPlaceDoc,
						&m_rcPos, &rcClip, &m_frameInfo)))
		{
			ASSERT(m_pInPlaceFrame != NULL);

			CRect rectClip;
			if (!m_bInPlaceSiteWndless)
			{
				_GetClippingCoordinates(&m_rcPos, &rcClip, rectClip,
					&m_ptOffset);
				m_bInPlaceActive = CreateControlWindow(hwndParent, m_rcPos,
					rectClip);
			}
			else
			{
				m_bInPlaceActive = TRUE;
			}

			if (m_bInPlaceActive)
			{
				if (bUIActivate)
				{
					if (m_bInPlaceSiteEx)
					{
						if (m_pInPlaceSiteEx->RequestUIActivate() != S_OK)
							m_pInPlaceSite->OnUIDeactivate(FALSE);
					}

					BuildSharedMenu();

					m_bUIActive = TRUE;
					m_pInPlaceSite->OnUIActivate();

					m_pInPlaceFrame->SetActiveObject(
						&m_xOleInPlaceActiveObject, NULL);
					if (m_pInPlaceDoc != NULL)
						m_pInPlaceDoc->SetActiveObject(
							&m_xOleInPlaceActiveObject, NULL);

					if (m_hWnd != NULL)
					{
						BOOL bHandles = AmbientShowGrabHandles();
						BOOL bHatching = AmbientShowHatching();

						/* JAM -Replaced this...

						if (bHandles || bHatching)
							CreateTracker(bHandles, bHatching, rcClip);

						... with the following: */

						CRect rc;
						GetRectInContainer(&rc);
						SetRectInContainer(CRect(0, 0, rc.Width(), rc.Height()));
					}

					AddFrameLevelUI();

					if (bUIActivate != -1 &&
						(m_hWnd != NULL) && !IsChild(GetFocus()))
					{
						SetFocus();
					}
				}

				// Pass thru the window message that caused us to be activated
				if ((m_hWnd != NULL || m_bInPlaceSiteWndless) && (pMsg != NULL))
					ForwardActivationMsg(pMsg);

				// Send appropriate notifications...
				SendAdvise(OBJECTCODE_SHOWOBJECT);

				return S_OK;
			}
		}
	}

	RELEASE(m_pInPlaceFrame);
	RELEASE(m_pInPlaceDoc);

	return E_FAIL;
}

